#!/usr/bin/env python3
##
## -----------------------------------------------------------------
##    This file is part of WAPT Software Deployment
##    Copyright (C) 2012 - 2022  Tranquil IT https://www.tranquil.it
##    All Rights Reserved.
##
##    WAPT helps systems administrators to efficiently deploy
##    setup, update and configure applications.
## ------------------------------------------------------------------
##

import os
import json
import setuphelpers
from waptpackage import PackageRequest, PackageEntry


def self_service_rules(wapt):
    """Returns dict of allowed packages for users and groups
    """
    cur = wapt.waptdb.execute("""select package,persistent_dir from wapt_localstatus s where s.section='selfservice' and s.persistent_dir is not null""")
    result = {}
    for (package, persistent_dir) in cur.fetchall():
        if persistent_dir:
            rules_fn = setuphelpers.makepath(persistent_dir, 'selfservice.json')
            if os.path.isfile(rules_fn):
                with open(rules_fn, 'r') as f:
                    rules = json.load(f)
                for group, packages in rules.items():
                    if not group in result:
                        result[group.lower()] = packages
                    else:
                        group_packages = result[group.lower()]
                        for package in packages:
                            if not package in group_packages:
                                group_packages.append(package)
    return result


def authorized_packages_for_token(wapt, token, package_requests=None):
    token_gen = wapt.get_secured_token_generator()
    user_pac = token_gen.loads(token)
    user_groups = user_pac.get('groups', [])

    rules = self_service_rules(wapt)

    result = []
    if package_requests is None:
        for group in user_groups:
            for package in rules.get(group, []):
                if not package in result:
                    result.append(package)
    else:
        for package_request in package_requests:
            if isinstance(package_request, PackageRequest):
                pr = package_request
            elif isinstance(package_request, str):
                pr = PackageRequest(pr)
            elif isinstance(package_request, PackageEntry):
                pr = package_request.as_package_request()
            else:
                continue

            for group in user_groups:
                if pr.package in rules.get(group, []):
                    result.append(pr)
                    break

    return result
